@@ -1,8 +1,8 @@
 glmrob <-
-function (formula, family = binomial, data, weights, subset,
-		   na.action, start = NULL, offset, method = "Mqle",
-		   weights.on.x = c("none", "hat", "robCov"), control = NULL,
-		   model = TRUE, x = FALSE, y = TRUE, contrasts = NULL,	 ...)
+function (formula, family, data, weights, subset,
+	  na.action, start = NULL, offset, method = "Mqle",
+	  weights.on.x = c("none", "hat", "robCov", "covMcd"), control = NULL,
+	  model = TRUE, x = FALSE, y = TRUE, contrasts = NULL, ...)
 {
     call <- match.call()
     if (is.character(family))
@@ -15,12 +15,13 @@ function (formula, family = binomial, data, weights, subset,
     }
     if (!(family$family %in% c("binomial", "poisson"))) {
 	if(family$family == "gaussian")
-	    stop("Use rlm(formula, ...) from library MASS for the Gaussian case")
+	    ## FIXME: use  our	lmrob() as soon as its available
+	    stop("Use MASS::rlm(formula, ...) for the Gaussian case")
 	else
 	    stop("Robust fitting method not yet implemented for this family")
     }
     if(is.null(control)) # -> use e.g., glmrobMqle.control()
-	control <- get(paste("glmrob", method, ".control", sep = ""))()
+	control <- get(paste("glmrob", method, ".control", sep = ""))(...)
     if (missing(data))
 	data <- environment(formula)
     ##
@@ -40,7 +41,7 @@ function (formula, family = binomial, data, weights, subset,
     weights <- model.weights(mf)
     offset <- model.offset(mf)
     if (!is.null(weights) && any(weights < 0))
-	stop("Negative wts not allowed")
+	stop("'weights' must be non-negative")
     if (!is.null(offset) && length(offset) != NROW(Y))
 	stop("Number of offsets is ", length(offset), ", should equal ",
 	     NROW(Y), " (number of observations)")
@@ -62,43 +63,48 @@ function (formula, family = binomial, data, weights, subset,
     ##-		    y = Y, weights = weights, offset = offset, family = family,
     ##-		    control = control, intercept = TRUE)$deviance
     ##-	    }
+    fit$na.action <- attr(mf, "na.action")
     if (model)
 	fit$model <- mf
-    fit$na.action <- attr(mf, "na.action")
     if (x)
 	fit$x <- X
     if (!y)
 	fit$y <- NULL
-    fit <- c(fit, list(call = call, formula = formula, terms = mt,
-		       data = data, offset = offset, control = control, method = method,
-		       contrasts = attr(X, "contrasts"),
-		       xlevels = .getXlevels(mt, mf)))
+    fit <- c(fit,
+	     list(call = call, formula = formula, terms = mt, data = data,
+		  offset = offset, control = control, method = method,
+		  contrasts = attr(X, "contrasts"),
+		  xlevels = .getXlevels(mt, mf)))
     class(fit) <- c("glmrob", "glm")
     fit
 }
 
 
 
-summary.glmrob <- function (object, correlation=FALSE, symbolic.cor=FALSE, ...)
+summary.glmrob <- function(object, correlation=FALSE, symbolic.cor=FALSE, ...)
 {
     dispersion <- 1
     coefs <- object$coefficients
+    aliased <- is.na(coefs)# needs care; also used in print method
+    if(any(aliased))
+	coefs <- coefs[!aliased]
     covmat <- object$cov
-    dimnames(covmat) <- list(names(coefs), names(coefs))
     var.cf <- diag(covmat)
     s.err <- sqrt(var.cf)
     zvalue <- coefs/s.err
-    dn <- c("Estimate", "Std.Error")
-
     pvalue <- 2 * pnorm(-abs(zvalue))
-    coef.table <- cbind(coefs, s.err, zvalue, pvalue)
-    dimnames(coef.table) <- list(names(coefs),
-				 c(dn, "z-value", "Pr(>|z|)"))
-    ans <- c(object[c("call", "terms", "family", "iter", "method")],
-	     ## MM: maybe we should rather keep the full object?
+    coef.table <- cbind("Estimate" = coefs, "Std. Error" = s.err,
+			"z-value" = zvalue, "Pr(>|z|)" = pvalue)
+
+    ans <- c(object[c("call", "terms", "family", "iter", "control", "method",
+                      "residuals", "fitted.values")],
+	     ## MM: should rather keep more from 'object' ?
+	     ##	    currently, cannot even print the asympt.efficiency!
 	     list(deviance=NULL, df.residual=NULL, null.deviance=NULL,
-		  df.null=NULL, deviance.resid=NULL, coefficients=coef.table,
-		  dispersion=dispersion, df=NULL , cov.unscaled=covmat))
+		  df.null= NULL, df= NULL, ## (because of 0 weights; hmm,...)
+		  aliased = aliased,
+		  coefficients = coef.table, dispersion = dispersion,
+		  cov.unscaled = covmat))
     if (correlation) {
 	ans$correlation <- cov2cor(covmat)
 	ans$symbolic.cor <- symbolic.cor
@@ -107,6 +113,14 @@ summary.glmrob <- function (object, correlation=FALSE, symbolic.cor=FALSE, ...)
     return(ans)
 }
 
+## almost a copy of vcov.glm() [if that didn't have summmary.glm() explicitly]
+vcov.glmrob <- function (object, ...)
+{
+    so <- summary(object, corr = FALSE, ...)
+    so$dispersion * so$cov.unscaled
+}
+
+
 print.glmrob <- function (x, digits = max(3, getOption("digits") - 3), ...)
 {
     cat("\nCall: ", deparse(x$call), "\n\n")
@@ -120,23 +134,31 @@ print.glmrob <- function (x, digits = max(3, getOption("digits") - 3), ...)
 		      print.gap = 2, quote = FALSE)
     }
     else cat("No coefficients\n\n")
-    cat("\nNumber of oberservations:", length(x$y),
-	"\nFitted by method = ", x$method, "\n")
+    cat("\nNumber of observations:", length(x$residuals),
+	"\nFitted by method ", sQuote(x$method), "\n")
     invisible(x)
 }
 
-
 print.summary.glmrob <-
     function (x, digits = max(3, getOption("digits") - 3),
 	      symbolic.cor = x$symbolic.cor,
 	      signif.stars = getOption("show.signif.stars"), ...)
 {
-    cat("\nCall:\n")
-    cat(paste(deparse(x$call), sep="\n", collapse="\n"), "\n\n", sep="")
-    cat("\nCoefficients:\n")
-    coefs <- x$coefficients
-    printCoefmat(coefs, digits = digits, signif.stars = signif.stars,
-		 na.print = "NA", ...)
+    cat("\nCall: ", deparse(x$call), "\n\n")
+    if (length(cf <- coef(x))) {
+	if(nsingular <- sum(x$aliased)) # glm has   df[3] - df[1]
+	    cat("\nCoefficients: (", nsingular,
+		" not defined because of singularities)\n", sep = "")
+	else cat("\nCoefficients:\n")
+	printCoefmat(cf, digits = digits, signif.stars = signif.stars,
+		     na.print = "NA", ...)
+    }
+    else cat("No coefficients\n\n")
+
+    n <- length(x$residuals)
+    cat("\nNumber of observations:", n,
+	"\nFitted by method", sQuote(x$method)," (in", x$iter, "iterations)\n")
+
     cat("\n(Dispersion parameter for ", x$family$family,
 	" family taken to be ", format(x$dispersion), ")\n\n",sep = "")
     if(any(!is.null(unlist(x[c("null.deviance", "deviance")]))))
@@ -146,17 +168,15 @@ print.summary.glmrob <-
 			       digits=max(5, digits + 1)), " on",
 			format(unlist(x[c("df.null", "df.residual")])),
 			" degrees of freedom\n"), 1, paste, collapse=" "),
-	    "\n\n", sep = "")
+	    "\n", sep = "")
     else
-	cat("No deviance values available \n\n")
-    cat("Fitted by method = ", x$method, "\n",
-	"Number of iterations: ", x$iter, "\n", sep = "")
+	cat("No deviance values available \n")
     correl <- x$correlation
     if (!is.null(correl)) {
 	p <- NCOL(correl)
 	if (p > 1) {
 	    cat("\nCorrelation of Coefficients:\n")
-	    if (is.logical(symbolic.cor) && symbolic.cor) {
+	    if (isTRUE(symbolic.cor)) {
 		print(symnum(correl, abbr.col=NULL))
 	    }
 	    else {
